from openapi.providers.sms.wgws import Client

from examples.config import config

client = Client(**config['sms']['wgws'])
client.add_webhook(config['openapi_webhook'])

content = ''
print(content)
# For testing
result = client.request(
    'post', '/wgws/OrderServlet',
    data={'content': content, 'calledNumber': f"{config['sms']['to']}"}
)
print(result)
print(result.error == client.codes.SUCCESS)



from typing import Dict
from openapi.providers.wechat.open import Client, Result

from examples.config import config


class OrderDetailResult(Result):
    order: Dict


client = Client(**config['wechat_channels'])
# client.add_webhook(config['openapi_webhook'])
result = client.request(
    'POST', endpoint='/channels/ec/order/get',
    data={
        'order_id': '3713399872155682304'
    }, is_oauth=False, result_processor=OrderDetailResult
)
print(result)



from typing import Optional, Union, List, Dict
from pydantic import Field

from openapi.enums import IntegerChoices
from openapi.providers.base import BaseClient, BaseResult


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'


class Result(BaseResult):
    status_code: Optional[int] = Field(alias='StatusCode')


class Client(BaseClient):
    NAME = '飞书 - 群组机器人'
    API_BASE_URL = 'https://open.feishu.cn/open-apis/bot/'
    API_VERSION = 'v2'

    def __init__(self, secret=None):
        super().__init__()
        self.secret = secret
        self.codes = Code

    def fetch_access_token(self):
        pass

    def request(
        self,
        method,
        endpoint,
        json: Optional[Union[Dict, List]] = None,
    ):
        request_url = f'{self.API_BASE_URL}{self.API_VERSION}{endpoint}'
        response = self._request(method, request_url, json=json)
        return Result(**(response.json() if response else {'code': self.codes.FAIL}))



import base64
import socket
import struct
import hashlib

from Crypto.Cipher import AES

from openapi.enums import IntegerChoices
from openapi.exceptions import DisallowedHost
from openapi.providers.base import BaseClient, BaseResult, Token
from openapi.utils import xml_to_dict


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'
    INVALID_WHITE_LIST = 2051, 'ip 未在白名单'


class Result(BaseResult):
    msg: str = ''


class Client(BaseClient):
    NAME = '小鹅通'
    API_VERSION = ''
    API_BASE_URL = 'https://api.xiaoe-tech.com'

    def __init__(self, app_id, secret, client_id=None, decrypt_key=None, decrypt_token=None):
        super().__init__()
        self.codes = Code

        self.app_id = app_id
        self.secret = secret
        self.client_id = client_id

        self.decrypt_key = decrypt_key
        self.decrypt_token = decrypt_token

    def request(self, method, endpoint, params=None, data=None, token_request=False) -> Result:
        if not token_request:
            if method == 'get' and params and 'access_token' not in params:
                params['access_token'] = self.access_token

            if method == 'post' and data and 'access_token' not in data:
                data['access_token'] = self.access_token

        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, params=params, json=data)
        return Result(**response.json()) if response else Result(code=self.codes.FAIL)

    def check_token(self, access_token):
        result = self.request(
            'post', '/xe.user.batch.get/1.0.0', data={'access_token': access_token, 'page': 1, 'page_size': 1}
        )
        return result.code == self.codes.SUCCESS

    def fetch_access_token(self):
        result: Result = self.request(
            'get',
            '/token',
            params={
                'app_id': self.app_id,
                'secret_key': self.secret,
                'client_id': self.client_id,
                'grant_type': 'client_credential',
            },
            token_request=True,
        )
        if result.code == self.codes.SUCCESS:
            self._token = Token(**result.data)

        if result.code == self.codes.INVALID_WHITE_LIST:
            raise DisallowedHost(result.msg)

    def decrypt(self, encrypt, timestamp, nonce, signature):
        data = [self.decrypt_token, timestamp, nonce, encrypt]
        data.sort()

        sign_aligo = hashlib.sha1()
        sign_aligo.update(''.join(data).encode())
        if signature != sign_aligo.hexdigest():
            return Result(code=self.codes.FAIL, msg='签名不一致')

        key = base64.b64decode(f'{self.decrypt_key}=')
        crypter = AES.new(key, AES.MODE_CBC, key[:16])

        plain_text = crypter.decrypt(base64.b64decode(encrypt))
        pad = ord(plain_text[-1:])
        content = plain_text[16:-pad]
        xml_length = socket.ntohl((struct.unpack('I', content[:4]))[0])
        xml_content = content[4 : xml_length + 4]
        return Result(code=self.codes.SUCCESS, msg='OK', data=xml_to_dict(f'<xml>{xml_content.decode()}</xml>'))



from unittest import TestCase

from openapi.providers.feishu.bot import Result as BotResult
from openapi.providers.feishu.open import Result as OpenResult, Token as OpenToken


class FeiShuBotTest(TestCase):

    def test_result(self):
        r = BotResult(**{'StatusCode': 200})
        assert r.status_code == 200


class FeiShuOpenTest(TestCase):

    def test_result(self):
        r = OpenResult()
        assert r.msg == ''

    def test_token(self):
        t = OpenToken(access_token='1', )
        assert t.access_token == '1'



from typing import Any
from json import dumps as json_dumps
from xml.etree import ElementTree as EleTree


def dict_to_xml(data):
    xml_list = ['<xml>']
    for k, v in data.items():
        if not v:
            continue

        if str(v).isdigit():
            xml_list.append(f'<{k}>{v}</{k}>')
        else:
            xml_list.append(f'<{k}><![CDATA[{v}]]></{k}>')
    xml_list.append('</xml>')
    return ''.join(xml_list)


def xml_to_dict(xml_string):
    try:
        return dict((child.tag, child.text) for child in EleTree.fromstring(xml_string))
    except EleTree.ParseError:
        return {}


def encode_json(json: Any) -> str:
    return json_dumps(json, ensure_ascii=False, separators=(',', ':'), allow_nan=False)



from datetime import datetime
from openapi.providers.xhs import Client

from examples.config import config

# print(config)
client = Client(
    app_id=config['xhs']['app_id'], secret=config['xhs']['secret'],
    user_id=config['xhs']['user_id'], seller_id=config['xhs']['seller_id']
)
client.add_webhook(config['openapi_webhook'])
order_id = config['xhs']['test']['order_id']

info = config['xhs']['test']['info']
info['expires_at'] = datetime.fromtimestamp(info['expires_at'] / 1000)

# r = client.request(
#     'post', '/ark/open_api/v3/common_controller',
#     action='oauth.getAccessToken',
#     data={
#         'code': config['xhs']['code']
#     }
# )
# print(r)

r = client.request(
    'post', '/ark/open_api/v3/common_controller',
    action='order.getOrderDetail',
    data={
        'orderId': 'P718870545250225361'
    }
)
address_id = r.data['openAddressId']
print(address_id)

# r = client.request(
#     'post', '/ark/open_api/v3/common_controller',
#     action='order.getOrderReceiverInfo',
#     data={
#         'accessToken': config['xhs']['token'],
#         'receiverQueries': {
#             'orderId': 'P718870545250225361',
#             'openAddressId': address_id
#         }
#     }
# )
# print(r)
# client.make_token(**info)

r = client.request(
    'post', '/ark/open_api/v3/common_controller',
    action='data.batchDecrypt',
    data={
        'baseInfos': [
            {
                'dataTag': order_id,
                'encryptedData': '#O/TUHCniXiCEbZKi06W0NE3aCHmiw6hlDxi1wabfz2U=#O/TUHCniXiCEbZKi06W0NAP/QQxFlmN4IBXB3iu5Csc1kGKQhS693psz9eci0XbEFlowN4IyD9VUDhL2eX2k5h2kTY53vwktNnxKVy83nWw=#2##'
            },
            {
                'dataTag': order_id,
                'encryptedData': '#O/TUHCniXiCEbZKi06W0NHqMLy7peAQiu8VRjWCmJVA=#O/TUHCniXiCEbZKi06W0NAP/QQxFlmN4IBXB3iu5Csc1kGKQhS693psz9eci0XbEV2DSoCHAYct0sQIvVGxQz6zf/OswYhtN4W9+jxWAnsc=#3##'
            },
        ],
        'actionType': 2, 'appUserId': ''
    }
)
print(r)




import secrets
import time
from base64 import decodebytes, encodebytes
from pathlib import Path
from typing import Dict, Optional, Union

from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5

from openapi.enums import TextChoices
from openapi.exceptions import NotFoundPath
from openapi.providers.base import BaseClient, BaseResult


def format_params(data):
    return '&'.join([f'{k}={data[k]}' for k in sorted(data) if data[k] and k != 'sign'])


def calculate_signature(unsigned_string, api_key):
    signer = PKCS1_v1_5.new(api_key)
    signature = signer.sign(SHA256.new(unsigned_string))
    # base64 编码，转换为unicode表示并移除回车
    return encodebytes(signature).decode('utf-8').replace('\n', '')


def verify_signature(params, api_key, charset='utf-8'):
    sign = params['sign']
    string = format_params(params)
    signer = PKCS1_v1_5.new(api_key)
    digest = SHA256.new()
    digest.update(string.encode(charset))
    return signer.verify(digest, decodebytes(sign.encode(charset)))


class Code(TextChoices):
    SUCCESS = '10000', 'OK'
    SIGN_FAIL = '401', '签名错误'
    FAIL = '500', '错误信息'


class Result(BaseResult):
    code: Code | str = Code.SUCCESS
    data: Dict | None = None
    message: str | None = None


class Client(BaseClient):
    NAME = '联想支付'
    VERSION = '1.0'
    API_BASE_URL = 'https://cloud-rest.lenovomm.com/cloud-intermodal-core/api/v1'

    def __init__(self, app_id, mch_id, private_key_path, public_key_path):
        super().__init__()
        self.app_id = app_id
        self.mch_id = mch_id
        self.codes = Code

        public_key_path = Path(public_key_path)
        if not public_key_path.exists():
            raise NotFoundPath(f'{public_key_path} not found')

        private_key_path = Path(private_key_path)
        if not private_key_path.exists():
            raise NotFoundPath(f'{private_key_path} not found')

        # 加载公钥
        with open(public_key_path) as fp:
            self.public_key = RSA.importKey(fp.read())

        # 加载私钥
        with open(private_key_path) as fp:
            self.private_key = RSA.importKey(fp.read())

    def request(self, endpoint: str, data: Dict, method: str = 'post') -> Result:
        public_params = {
            'mchId': self.mch_id,
            'appId': self.app_id,
            'nonce': secrets.token_hex(16),
            'timestamp': str(int(time.time() * 1000)),
            'version': self.VERSION,
            'signType': 'RSA2',
        }
        data.update(**public_params)

        data['sign'] = calculate_signature(format_params(data).encode(), self.private_key)
        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, json=data)
        return Result(**response.json())

    def check_signature(self, data: Dict) -> bool:
        data.setdefault('appId', self.app_id)
        data.setdefault('mchId', self.app_id)
        return bool(verify_signature(data, self.public_key))

    def fetch_access_token(self):
        pass



from openapi.providers.crm.yunduo import Client, SignType

from examples.config import config

yunduo_config = config['yunduo']

client = Client(company_id=yunduo_config['company_id'])
client.add_webhook(config['openapi_webhook'])

"""
code=200 data={'id': None} message=None msg='操作成功'
code=600 data=None message=None msg='根据crm中的唯一标识,此数据已经存在'
code=200 data={'fc_6363': 'S090', 'uniqueCustomerId': '62e0f95d3953fa0001370d51', 'saleUserName': '', 'saleUserRealName': '', 'recentSaleUserName': '', 'recentSaleUserRealName': '', 'firstBelongSaleUserName': '', 'firstBelongSaleUserRealName': ''} message=None msg='成功'
"""


mobile = '19999999999'


if __name__ == '__main__':
    # 客户数据同步
    result = client.request(
        'post', f'/api/dataSync/customer/{client.company_id}',
        sign_key=yunduo_config['token'], sign_type=SignType.SHA256,
        params={'router_id': client.company_id},
        json={
            'contactPhone': mobile,
            'gender': '1',
            'birthdate': '1989-05-28',
            'spreadCity': '深圳市',
            'customerSource': '官网',
            'customerName': '谷爱凌',
            'consultType': '官网',
            'remarks': '【官网】官网测试',
            'tag': '自动分',
            'distribute_name': '未知',
            'fc_7581': 'S011'
        }
    )
    print(result)

    # 通用数据同步
    result = client.request(
        'post',
        f'/external/clue/standardClueAccept/v2/{client.company_id}/61ed22bc8ab9950001daf9a0',
        sign_key=yunduo_config['general_token'], sign_type=SignType.MD5,
        params={'router_id': client.company_id},
        json={
            'mobile': mobile,
            'memo': '【官网】Mysql 开发基础入门',
            'consultType': '官网',
            'distribute_name': '未知'
        }
    )
    print(result)

    # 客户信息查询
    result = client.request(
        'post',
        f'/api/dataSync/customerInfo/{client.company_id}',
        sign_key=yunduo_config['token'], sign_type=SignType.SHA256,
        params={'router_id': client.company_id},
        json={
            'contactPhone': mobile,
            'includes': ['fc_6363']
        }
    )
    print(result)



from unittest import TestCase

from openapi.providers.alipay import Result as AliPayResult, Code as AliPayCode
from openapi.providers.aliyun import Result as AliYunResult, Code as AliYunCode


class AliPayTest(TestCase):

    def test_result(self):
        r = AliPayResult()
        assert r.code == AliPayCode.SUCCESS


class AliYunTest(TestCase):

    def test_result(self):
        r = AliYunResult(request_id='123')
        assert r.code == AliYunCode.SUCCESS
        assert r.request_id == '123'



import json
import hashlib
from pathlib import Path
from base64 import decodebytes, encodebytes
from datetime import datetime
from urllib.parse import quote_plus
from typing import Dict

import OpenSSL
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5

from openapi.enums import IntegerChoices
from openapi.exceptions import NotFoundPath
from openapi.providers.base import BaseClient, BaseResult


class Code(IntegerChoices):
    SUCCESS = 0, '成功'
    FAIL = -1, '失败'


class Result(BaseResult):
    code: Code = Code.SUCCESS


def calculate_signature(unsigned_string, api_key):
    signer = PKCS1_v1_5.new(api_key)
    signature = signer.sign(SHA256.new(unsigned_string))
    # base64 编码，转换为unicode表示并移除回车
    return encodebytes(signature).decode('utf-8').replace('\n', '')


class Client(BaseClient):
    NAME = '支付宝'
    API_BASE_URL = 'https://openapi.alipay.com/gateway.do'
    API_VERSION = '1.0'

    def __init__(
        self,
        app_id,
        app_private_key_path,
        app_cert_public_key_path,
        alipay_root_cert_path,
        alipay_cert_public_key_path,
        is_sandbox=False,
    ):
        super().__init__()
        if is_sandbox:
            self.API_BASE_URL = 'https://openapi-sandbox.dl.alipaydev.com/gateway.do'

        self.codes = Code
        self.app_id = app_id

        app_private_key_path = Path(app_private_key_path)
        if not app_private_key_path.exists():
            raise NotFoundPath(f'{app_private_key_path} not found')

        app_cert_public_key_path = Path(app_cert_public_key_path)
        if not app_cert_public_key_path.exists():
            raise NotFoundPath(f'{app_cert_public_key_path} not found')

        alipay_root_cert_path = Path(alipay_root_cert_path)
        if not alipay_root_cert_path.exists():
            raise NotFoundPath(f'{alipay_root_cert_path} not found')

        alipay_cert_public_key_path = Path(alipay_cert_public_key_path)
        if not alipay_cert_public_key_path.exists():
            raise NotFoundPath(f'{alipay_cert_public_key_path} not found')

        # 加载应用的私钥
        with open(app_private_key_path) as fp:
            self.app_private_key = RSA.importKey(fp.read())

        # 加载应用证书公钥
        with open(app_cert_public_key_path) as fp:
            self.app_cert_public_key = fp.read()

        # 加载阿里根证书
        with open(alipay_root_cert_path) as fp:
            self.alipay_root_cert = fp.read()

        # 加载ali证书公钥
        with open(alipay_cert_public_key_path) as fp:
            self.alipay_cert_public_key = fp.read()

    def fetch_access_token(self):
        pass

    def build_params(self, endpoint, data, notify_url=None, return_url=None):
        default = {
            'app_id': self.app_id,
            'method': endpoint,
            'charset': 'utf-8',
            'sign_type': 'RSA2',
            'timestamp': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'app_cert_sn': self.app_cert_sn,
            'alipay_root_cert_sn': self.root_cert_sn,
            'version': self.API_VERSION,
            'biz_content': data,
        }

        if notify_url is not None:
            default['notify_url'] = notify_url

        if return_url is not None:
            default['return_url'] = return_url
        return default

    def build_query_params(self, data):
        ordered_items = sorted(
            ((k, v if not isinstance(v, dict) else json.dumps(v, separators=(',', ':'))) for k, v in data.items())
        )
        unsigned_string = '&'.join(f'{k}={v}' for k, v in ordered_items)
        sign = calculate_signature(unsigned_string.encode('utf-8'), self.app_private_key)
        quoted_string = '&'.join(f'{k}={quote_plus(str(v).encode())}' for k, v in ordered_items)
        return f'{quoted_string}&sign={quote_plus(sign)}'

    def request(self, method, endpoint='', params=None, data=None):
        params = self.build_query_params(self.build_params(endpoint, params))
        response = self._request(method, f'{self.API_BASE_URL}?{params}')
        if response is not None:
            return Result(data=response.json()[f'{"_".join(endpoint.split("."))}_response'])
        else:
            return Result(code=self.codes.FAIL)

    @property
    def alipay_public_key(self):
        cert = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, self.alipay_cert_public_key.encode('ascii'))
        return RSA.importKey(OpenSSL.crypto.dump_publickey(OpenSSL.crypto.FILETYPE_PEM, cert.get_pubkey()).decode())

    @property
    def app_cert_sn(self):
        cert = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, self.app_cert_public_key.encode('ascii'))
        cert_issue = cert.get_issuer()
        name = f'CN={cert_issue.CN},OU={cert_issue.OU},O={cert_issue.O},C={cert_issue.C}'
        m = hashlib.md5()
        m.update(bytes(f'{name}{cert.get_serial_number()}', encoding='utf-8'))
        return m.hexdigest()

    @property
    def root_cert_sn(self):
        root_cert_sn = None
        for cert in (
            OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, c.encode('ascii'))
            for c in self.alipay_root_cert.split('\n\n')
        ):
            try:
                alg = cert.get_signature_algorithm()
            except ValueError:
                continue
            if b'rsaEncryption' in alg or b'RSAEncryption' in alg:
                cert_issue = cert.get_issuer()
                name = f'CN={cert_issue.CN},OU={cert_issue.OU},O={cert_issue.O},C={cert_issue.C}'
                m = hashlib.md5()
                m.update(bytes(f'{name}{cert.get_serial_number()}', encoding='utf8'))
                cert_sn = m.hexdigest()
                if not root_cert_sn:
                    root_cert_sn = cert_sn
                else:
                    root_cert_sn = f'{root_cert_sn}_{cert_sn}'
        return root_cert_sn

    def check_signature(self, data: Dict) -> bool:
        sign: str = data['sign']
        check_data = {k: v for k, v in data.items() if k not in ('sign', 'sign_type')}
        verify_data = sorted(
            ((k, v if not isinstance(v, dict) else json.dumps(v, separators=(',', ':'))) for k, v in check_data.items())
        )
        message = '&'.join(f'{k}={v}' for k, v in verify_data)
        signer = PKCS1_v1_5.new(self.alipay_public_key)
        digest = SHA256.new()
        digest.update(message.encode())
        return bool(signer.verify(digest, decodebytes(sign.encode())))



import datetime
import time

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.hashes import MD5, Hash

from openapi.enums import IntegerChoices
from openapi.providers.base import BaseClient, BaseResult, Token


def calculate_signature(params, api_key):
    data = f'appId={params["appId"]}&timestamp={params["timestamp"]}&version={params["version"]}'
    h = Hash(algorithm=MD5(), backend=default_backend())
    h.update(f'{params["method"]}?{data}{api_key}'.encode())
    return h.finalize().hex()


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'
    INVALID_WHITE_LIST = 2051, 'ip 未在白名单'


class Result(BaseResult):
    success: bool
    error_code: int | None = None
    error_msg: str | None = None


class Client(BaseClient):
    NAME = '小红书'
    API_VERSION = '2.0'
    API_BASE_URL = 'https://ark.xiaohongshu.com'

    def __init__(self, app_id, secret, user_id, seller_id, redirect_url=''):
        super().__init__()
        self.app_id = app_id
        self.secret = secret
        self.user_id = user_id
        self.seller_id = seller_id
        self.redirect_url = redirect_url
        self.codes = Code

    def request(self, method, endpoint, action=None, params=None, data=None, token_request=False, headers=None):
        if data is not None:
            public_params = {
                'appId': self.app_id,
                'version': self.API_VERSION,
                'timestamp': str(int(time.time() * 1000)),
                'method': action,
            }
            public_params['sign'] = calculate_signature(public_params, self.secret)
            data.update(**public_params)
            if 'accessToken' not in data and not token_request:
                data['accessToken'] = self.access_token

        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, params=params, json=data, headers=headers)
        return Result(**response.json())

    def check_token(self, access_token=None):
        return self._token.is_valid

    def fetch_access_token(self):
        code_result = self.request(
            'get',
            '/api/edith/openapi/getcode',
            params={
                'appId': self.app_id,
                'userId': self.user_id,
                'sellerId': self.seller_id,
                'subsystemAlias': 'ark',
                'redirectUrl': self.redirect_url,
            },
        )
        code = None
        if code_result.code == Code.SUCCESS and code_result.success:
            code = code_result.data['code']

        if code is None:
            return

        result = self.request(
            'post',
            '/ark/open_api/v3/common_controller',
            action='oauth.getAccessToken',
            data={'code': code},
            token_request=True,
        )
        if result.error_code == Code.SUCCESS and result.success:
            self._token = Token(
                access_token=result.data['accessToken'],
                refresh_token=result.data['refreshToken'],
                expires_in=result.data['accessTokenExpiresAt'] / 1000 - time.time(),
                expires_at=datetime.datetime.fromtimestamp(result.data['accessTokenExpiresAt'] / 1000),
            )

    def refresh_access_token(self):
        result = self.request(
            'post',
            '/ark/open_api/v3/common_controller',
            action='oauth.refreshToken',
            data={'refreshToken': self._token.refresh_token},
            token_request=True,
        )
        if result.error_code == Code.SUCCESS and result.success:
            self._token = Token(
                access_token=result.data['accessToken'],
                refresh_token=result.data['refreshToken'],
                expires_in=result.data['accessTokenExpiresAt'] / 1000 - time.time(),
                expires_at=datetime.datetime.fromtimestamp(result.data['accessTokenExpiresAt'] / 1000),
            )



from openapi.providers.sms.submail import Client

from examples.config import config

client = Client(**config['sms']['submail'])
client.add_webhook(config['openapi_webhook'])

content = """【路飞学城】您已成功购买黑金VIP实战课，请联系您的专属班主任，领取代码课件，点击添加：http://link.luffycity.com/{link} 。
观看步骤：
1、请用下单手机号，电脑端登录路飞学城官网：www.luffycity.com（推荐使用谷歌浏览器）；
2、登录后在实战课页面，可看到黑金vip专属权限；
3、点击去学习按钮，即可观看实战课中所有的课程视频；
退订回N。""".format(link='123456')

# For testing
result = client.request(
    'post', '/message/send.json',
    json={'content': content, 'to': config['sms']['to']}
)
print(result)
print(result.status == client.codes.SUCCESS)



import time
import hashlib
import secrets
from pathlib import Path
from typing import Dict, Optional

from openapi.enums import TextChoices
from openapi.providers.base import BaseClient, BaseResult
from openapi.utils import dict_to_xml, xml_to_dict


class Code(TextChoices):
    SUCCESS = 'SUCCESS', '成功'
    FAIL = 'FAIL', '失败'


class Result(BaseResult):
    return_code: str
    return_msg: str
    result_code: str | None = None
    data: Dict | None = None


def calculate_signature(params, api_key, sign_type='MD5'):
    if sign_type == 'MD5':
        h = hashlib.md5()
    else:
        h = hashlib.md5()

    data = [f'{k}={params[k]}' for k in sorted(params) if params[k]]
    if api_key:
        data.append(f'key={api_key}')
    h.update(bytes('&'.join(data), encoding='utf-8'))
    return h.hexdigest().upper()


class Client(BaseClient):
    NAME = '微信支付'
    API_BASE_URL = 'https://api.mch.weixin.qq.com'
    API_VERSION = ''

    def __init__(self, app_id, mch_id, api_key_path, is_sandbox):
        super().__init__()
        self.app_id = app_id
        self.mch_id = mch_id
        self.api_key_path = api_key_path
        self.is_sandbox = is_sandbox

        self.codes = Code

        with open(Path(api_key_path)) as fd:
            self.api_key = fd.read().replace('\r\n', '')

    @property
    def debug_api_key(self):
        return ''

    def request(self, method: str, endpoint: str, params: Optional[Dict] = None, data: Optional[Dict] = None):
        request_url = f'{self.API_BASE_URL}{"/sandboxnew" if self.is_sandbox else ""}{endpoint}'

        if data:
            data['appid'] = self.app_id
            data['mch_id'] = self.mch_id

            if 'nonce_str' not in data:
                data['nonce_str'] = secrets.token_hex(16)

            data['sign'] = calculate_signature(data, self.debug_api_key if self.is_sandbox else self.api_key)

        response = self._request(method, request_url, params=params, data=dict_to_xml(data).encode('utf-8'))
        if response is None:
            return Result(return_code=self.codes.FAIL)
        result = xml_to_dict(response.content)
        return Result(
            return_msg=result['return_msg'],
            return_code=result['return_code'],
            result_code=result.get('result_code'),
            data=result,
        )

    def get_jsapi_data(self, prepay_id):
        data = {
            'appId': self.app_id,
            'timeStamp': str(int(time.time())),
            'nonceStr': secrets.token_hex(16),
            'signType': 'MD5',
            'package': f'prepay_id={prepay_id}',
        }
        sign = calculate_signature(data, self.api_key if not self.is_sandbox else self.debug_api_key)
        data['paySign'] = sign.upper()
        return data

    def check_signature(self, data: Optional[Dict]) -> bool:
        return data['sign'] == calculate_signature(
            {k: v for k, v in data.items() if k not in ('sign',)},
            self.api_key if not self.is_sandbox else self.debug_api_key,
        )

    def fetch_access_token(self):
        pass



import json
from datetime import datetime
from typing import Dict, Optional

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.hashes import MD5, SHA256, Hash
from cryptography.hazmat.primitives.hmac import HMAC

from openapi.enums import IntegerChoices
from openapi.providers.base import BaseClient, BaseResult, Token

SIGN_FIELDS = ['app_key', 'method', 'param_json', 'timestamp', 'v']


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 10000, '成功'


def format_params(params, secret=None):
    data = [f'{k}{params[k]}' for k in params if k in SIGN_FIELDS]
    return f'{secret}{"".join(sorted(data))}{secret}'


def calc_signature(params, secret):
    pattern = format_params(params, secret).encode('utf-8')
    hmac = HMAC(key=secret.encode('utf-8'), algorithm=SHA256(), backend=default_backend())
    hmac.update(pattern)
    signature = hmac.finalize()
    return signature.hex()


class Result(BaseResult):
    log_id: str | None = None
    msg: str | None = None
    sub_code: str | None = None
    sub_msg: str | None = None


class Client(BaseClient):
    NAME = '抖店'
    API_BASE_URL = 'https://openapi-fxg.jinritemai.com/'
    API_VERSION = 2

    def __init__(self, app_id, secret, shop_id):
        super().__init__()

        self.app_id = app_id
        self.secret = secret
        self.shop_id = shop_id
        self.codes = Code

    def request(self, method, endpoint, params=None, data=None, token_request=False) -> Result:
        if params is None:
            params = {
                'v': self.API_VERSION,
                'app_key': self.app_id,
                'timestamp': datetime.now().strftime('%Y-%m-%d %X'),
                'method': '.'.join(filter(lambda _: bool(_), endpoint.split('/'))),
                'param_json': json.dumps(
                    {key: value for key, value in data.items() if value is not None},
                    sort_keys=True,
                    separators=(',', ':'),
                ),
            }
        sign = calc_signature(params, self.secret)
        params['sign_method'] = 'hmac-sha256'
        params['sign'] = sign
        if not token_request:
            params['access_token'] = self.access_token

        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, params=params, data=data)
        return Result(**response.json()) if response else Result(code=self.codes.FAIL)

    def fetch_access_token(self):
        result = self.request(
            'post',
            '/token/create',
            data={'code': '', 'grant_type': 'authorization_self', 'shop_id': self.shop_id},
            token_request=True,
        )
        if result.code == self.codes.SUCCESS:
            self._token = Token(**result.data)

    def refresh_access_token(self):
        result = self.request(
            'post',
            '/token/create',
            data={
                'refresh_token': self._token.refresh_token,
                'grant_type': 'refresh_token',
            },
        )
        if result.code == self.codes.SUCCESS:
            self._token = Token(**result.data)

    def callback(self, app_id: str, sign: str, body: bytes) -> Optional[Dict]:
        data = body.decode('utf-8')
        if not data:
            return

        if app_id != self.app_id:
            return

        h = Hash(algorithm=MD5(), backend=default_backend())
        h.update(f'{app_id}{data}{self.secret}'.encode())
        if h.finalize().hex() != sign:
            return

        return json.loads(data)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2023/2/3

import json
from openapi.providers.lenovo import Client
from examples.config import config

lenovo_pay_config = config['lenovopay']

pay_data = {
    'mchNo': '397120230402453989',
    'payAmount': 1,
    'goodsName': 'table',
    'goodsDec': '',
    'goodsCode': '1',
    'productCode': 'COLLECT_CODE',
    'attach': json.dumps({'mobile': 18803561683}),
    'payNotify': 'http://47.94.238.32:8000/api/v1/pay/lenovo/'
}

client = Client(**lenovo_pay_config)
result = client.request('/pay/trade/direct/link', data=pay_data)
print(result)



import httpx

from openapi.providers.alipay import Client

from examples.config import config

settings = config['alipay-test']
client = Client(**settings, is_sandbox=True)
# client.add_webhook(config['openapi_webhook'])


if __name__ == '__main__':
    # query
    params = client.build_query_params(client.build_params(
        'alipay.data.bill.balance.query', data={},
        # 'alipay.data.bill.balancehis.query', data={'biz_month': '2023-05'}
        # 'alipay.data.dataservice.bill.downloadurl.query',
        # data={'bill_type': 'trade', 'bill_date': '2023-05'}
    ))
    request_url = f'{client.API_BASE_URL}?{params}'
    print(request_url)
    response = httpx.get(request_url)
    print(response.json())
    # pc-pay
    pc_pay_params = client.build_query_params(client.build_params(
        'alipay.trade.page.pay',
        {
            'subject': 'popmart-molly',
            'out_trade_no': 'pc1234567',
            'total_amount': '0.01',
            'product_code': 'FAST_INSTANT_TRADE_PAY'
        },
        notify_url='http://47.94.172.250:9527/api/v1/pay/alipay/',
        return_url='http://47.94.172.250:9527/api/v1/pay/alipay/'
    ))
    pc_pay_url = f'{client.API_BASE_URL}?{pc_pay_params}'
    # print(pc_pay_url)

    # mobile-pay
    mobile_pay_params = client.build_query_params(client.build_params(
        'alipay.trade.wap.pay',
        {
            'subject': 'popmart-molly',
            'out_trade_no': 'mobile1234567',
            'total_amount': '0.01',
            'product_code': 'QUICK_WAP_WAY'
        },
        notify_url='http://47.94.172.250:9527/api/v1/pay/alipay/',
        return_url='http://47.94.172.250:9527/api/v1/pay/alipay/'
    ))
    mobile_pay_url = f'{client.API_BASE_URL}?{mobile_pay_params}'
    print(mobile_pay_url)

    # query
    result = client.request(
        'get', 'alipay.trade.query',
        params={
            # 'out_trade_no': '2023021922001494131437680153',
            'trade_no': '2023040322001494481429840718'
        }
    )
    print(result)



import uuid
import datetime
from typing import Dict, Optional

from openapi.enums import IntegerChoices
from openapi.providers.base import BaseClient, BaseResult
from openapi.utils import xml_to_dict


class Code(IntegerChoices):
    SUCCESS = 0, '成功'
    FAIL = -1, '失败'


class Result(BaseResult):
    error: int
    message: str | None = None


class Client(BaseClient):
    NAME = '中网信'
    API_BASE_URL = 'http://139.224.36.226:382'
    API_VERSION = ''

    def __init__(self, app_id, app_key):
        super(Client, self).__init__()
        self.app_id = app_id
        self.app_key = app_key

        self.codes = Code

    def request(self, method, endpoint, params: Optional[Dict] = None, data: Optional[Dict] = None):
        default_data = {
            'apName': self.app_id,
            'apPassword': self.app_key,
            'sendTime': datetime.datetime.now().strftime('%Y%m%d%H%M%S'),
            'serviceId': {'orgMsgId': uuid.uuid4().hex},
            'srcId': '123',
        }
        default_data.update(**data)
        request_url = f'{self.API_BASE_URL}{endpoint}'
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        response = self._request(method, request_url, params, default_data, headers=headers)
        return Result(**(xml_to_dict(response.content) if response else {'status': self.codes.FAIL}))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2023/4/2
import json
from openapi.providers.lenovo import Client
from examples.config import config

lenovo_pay_config = config['lenovopay']
client = Client(**lenovo_pay_config)
result = client.request('/pay/query/trade', data={
    'mchNo': '397120230402453989'
})
print(result)



from openapi.providers.yizhi import Client

from examples.config import config

# print(config)
client = Client(**config['yizhi'])
client.add_webhook(config['openapi_webhook'])

result = client.request(
    'post', '/orderList/v2',
    data={
        'page': '1', 'type': '1',
        'out_trade_no': '40484728108498653184',
        # 'trade_no': '40484728108498653184'
    }
)
# print(result)

result = client.request(
    'post', '/curriculumList',
    data={
        'page': '1', 'page_size': '100'
    }
)
print(result.data)

result = client.request(
    'post', '/spreadList',
    data={
        'page': '1', 'page_size': '100'
    }
)
# print(result.data['data'])

data = {
    'encrypt': 'MRVrmKNDi4ip65bRKLNVzlUYr+s3iI40WzY4fPxpGDlDjV93/v2UEa4KvuyPpGiOJKem7yNYwc6R80qv2zjRGiY8+xqfu5EqW1mo6VSVlFkT3He5Qmj/8TRL+F5YMewhiUOd5iCaZBLnMnfxTqAtFK5GsiHkHnvOAzs12C6WSHLHYwmgHoyRdrP5ZZhsIm3VAqnIlpBQ7UBLKCfnbkyZJoYkq4dL1R6/6onpFXhGI1qWlItJp/6dGKMoPHB7nzVq5XI0lkmNr64tK8PkgfsmF1UvRJoKpgza3zkOx2lAZ+I/OgJ36vYeMUt9HnqvQJRnuZ6YugoWhCwF10c/4ypFeLL000aNok9kh4Sllu2Dma/3hPi9/mqolCWgTTQbIetskvcnkMsMpcTRpQjkxvqaYMoS1dM3k+335UI1PO/EsrEBGLTjreg/Vu2N3/I37Vgif7Zs+lTVMNBGXJFK1KirVnjnbcU6RngQ3qsyBpRhgGkuyvVPMkuN7g5Q6YF+qneg',
    'timestamp': '1670063928',
    'nonce': '0g2EGsVR',
    'signature': '85f46e73437a94f7607e8ed08ce0f042ed3cb403'
}

result = client.decrypt(**data)
# print(result.data['data']['spread_id'])
# agent_id = result.data['agent_id']



import hmac
import time
import hashlib
from typing import Dict, Optional

from openapi.enums import IntegerChoices, TextChoices
from openapi.providers.base import BaseClient, BaseResult


class SignType(TextChoices):
    MD5 = 'MD5', 'MD5'
    SHA256 = 'SHA256', 'SHA256'


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 200, '操作成功'
    REPEATED = 600, '重复的数据'


class Result(BaseResult):
    msg: str = ''


def calc_signature(params: Dict, company_id: str, sign_key: str, timestamp: int, sign_type: SignType):
    if sign_type == SignType.SHA256:
        return hmac.new(f'{timestamp}{company_id}'.encode(), sign_key.encode(), digestmod=hashlib.sha256).hexdigest()

    if sign_type == SignType.MD5:
        sorted_params = [f'{key}={params[key]}' for key in sorted(params) if params[key]]
        sign_string = f'{"&".join(sorted_params)}{sign_key}{timestamp}'
        h = hashlib.md5()
        h.update(sign_string.encode())
        return h.hexdigest()


class Client(BaseClient):
    NAME = '云朵'
    API_BASE_URL = 'https://outcrmapi.yunduocrm.com'
    API_VERSION = ''

    def __init__(self, company_id):
        super().__init__()

        self.company_id = company_id
        self.codes = Code

    def fetch_access_token(self):
        pass

    def request(
        self,
        method,
        endpoint,
        sign_key,
        sign_type: SignType,
        params: Optional[Dict] = None,
        data: Optional[Dict] = None,
        json: Optional[Dict] = None,
    ):
        request_url = f'{self.API_BASE_URL}{endpoint}'
        timestamp = int(time.time())
        default_headers = {'timestamp': f'{timestamp}'}
        if sign_type == SignType.MD5:
            default_headers.update(
                token=sign_key, sign=calc_signature(json, self.company_id, sign_key, timestamp, sign_type)
            )
        else:
            default_headers.update(
                accessToken=sign_key, signature=calc_signature(json, self.company_id, sign_key, timestamp, sign_type)
            )
        response = self._request(method, request_url, params, data, json, headers=default_headers)
        return Result(**(response.json() if response else {'code': self.codes.FAIL}))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2022/11/14
import json

from openapi.providers.wechat.pay import Client

from examples.config import config

wechatpay_config = config['wechatpay2']

wxpay_api = Client(
    app_id=wechatpay_config['app_id'],
    mch_id=wechatpay_config['mch_id'],
    api_key_path=wechatpay_config['api_key_path'],
    is_sandbox=False
)

# 查询订单
result = wxpay_api.request(
    'post', '/pay/orderquery',
    data={
        # 'out_trade_no': '9wfafGR31rCytY68wSFcXCII',
        'transaction_id': '4200001591202211145504991626'
    }
)
print(result)

data = {
    'body': '米诺地尔町',
    'out_trade_no': '1232112359910',
    'total_fee': 1,
    'spbill_create_ip': '127.0.0.1',
    'notify_url': 'htt',
    'trade_type': 'NATIVE',
    'attach': json.dumps({'x': 1})
}

# 创建订单 -> pc
result = wxpay_api.request('post', '/pay/unifiedorder', data=data)
print(result)

# 创建订单 -> mweb
# data.update(trade_type='MWEB', out_trade_no='1231asd1222')
# result = wxpay_api.request('post', '/pay/unifiedorder', data=data)
# print(result)

# 创建订单 -> jsapi
# data.update(trade_type='JSAPI', out_trade_no='21321asd12311313', openid='ofwIAuEgpTZZwdPc1aort93xOdU8')
# result = wxpay_api.request('post', '/pay/unifiedorder', data=data)
# print(result)
# if result.result_code == wxpay_api.codes.SUCCESS:
#     jsapi_data = wxpay_api.get_jsapi_data(result.data['prepay_id'])
#     print(jsapi_data)



import json as _json
from datetime import datetime, timedelta
from typing import Dict, List, Optional, Union

import httpx
from pydantic import BaseModel, Field

MESSAGE_TEMPLATE = """date: {date}
method: {method}
request-url: {endpoint}
headers: 
{headers}
params: 
{params}
data:
{data}
result:
{result}
is_error: {is_error}
errmsg: {errmsg}
"""

SENSITIVE_KEYS = ['app_key', 'secret', 'secret_key', 'signature']


def mask_sensitive_data(data):
    if type(data) != dict:
        return data

    for key, value in data.items():
        if key in SENSITIVE_KEYS:
            data[key] = '***FILTERED***'

        if type(value) == dict:
            data[key] = mask_sensitive_data(data[key])

        if type(value) == list:
            data[key] = [mask_sensitive_data(item) for item in data[key]]

    return data


class Token(BaseModel):
    access_token: str
    expires_in: int = 7200
    refresh_token: Optional[str] = None
    created_at: datetime = Field(default_factory=datetime.now)
    expires_at: Optional[datetime] = None
    refresh_expires_at: Optional[datetime] = None

    @property
    def is_valid(self):
        if self.expires_at is not None:
            return datetime.now() < (self.expires_at - timedelta(minutes=20))
        return (datetime.now() - self.created_at).total_seconds() < (self.expires_in - self.expires_in * 0.3)


class BaseResult(BaseModel):
    code: int | None = None
    data: Union[List, Dict, int] | None = None
    message: str | None = None


class BaseClient:
    NAME = 'Unknown Client'
    API_BASE_URL = ''
    API_VERSION = ''

    def __init__(self):
        self._ignore_endpoints = []
        self._token: Optional[Token] = None
        self.enable_webhook = False
        self.webhook_url = None

    def _request(self, method, request_url, params=None, data=None, json=None, headers=None) -> httpx.Response:
        response = None
        is_error = False
        errmsg = ''
        try:
            response = httpx.request(
                method, request_url, headers=headers, params=params, data=data, json=json, timeout=httpx.Timeout(10)
            )
            is_error = response.is_error
        except httpx.HTTPError as exc:
            is_error = True
            errmsg = str(exc)
        finally:
            _data = data or json or {}
            format_data = {
                'date': datetime.now().strftime('%Y-%m-%d %X'),
                'method': method,
                'endpoint': request_url,
                'headers': _json.dumps(mask_sensitive_data(headers or {}), indent=2, ensure_ascii=False),
                'params': _json.dumps(mask_sensitive_data(params or {}), indent=2, ensure_ascii=False),
                'data': _json.dumps(mask_sensitive_data(_data), indent=2, ensure_ascii=False)
                if isinstance(_data, dict)
                else _data,
                'result': response.text if response is not None else '',
                'is_error': is_error,
                'errmsg': errmsg,
            }
            endpoint = request_url.replace(self.API_BASE_URL, '')
            if self.enable_webhook and self.webhook_url and endpoint not in self._ignore_endpoints:
                self.send_webhook_message(MESSAGE_TEMPLATE.format(**format_data))

        return response

    def fetch_access_token(self):
        raise NotImplementedError()

    def request(self, *args, **kwargs):
        raise NotImplementedError()

    def send_webhook_message(self, message):
        try:
            httpx.request(
                'post',
                self.webhook_url,
                json={
                    'msg_type': 'post',
                    'content': {
                        'post': {
                            'zh_cn': {
                                'title': self.NAME,
                                'content': [
                                    [{'tag': 'text', 'text': message}],
                                ],
                            }
                        }
                    },
                },
            )
        except httpx.HTTPError:
            pass

    def check_token(self, access_token=None):
        return True

    def refresh_access_token(self):
        pass

    def make_token(self, access_token, refresh_token, expires_in=7200, expires_at: datetime = None):
        if expires_at:
            expires_in = (expires_at - datetime.now()).seconds

        self._token = Token(
            access_token=access_token, refresh_token=refresh_token, expires_in=expires_in, expires_at=expires_at
        )

    @property
    def access_token(self):
        if not (self._token and self._token.is_valid and self.check_token(self._token.access_token)):
            self.fetch_access_token()

        return self._token.access_token

    def add_webhook(self, webhook_url, ignore_endpoints: Optional[List] = None):
        self.enable_webhook = True
        self.webhook_url = webhook_url
        if ignore_endpoints:
            self._ignore_endpoints = ignore_endpoints
